@@ -1,20 +1,20 @@
 // -*- C++ -*-
-/* This file is part of
- * ======================================================
+/**
+ * \file src/Spacing.h
+ * This file is part of LyX, the document processor.
+ * Licence details can be found in the file COPYING.
  *
- *           LyX, The Document Processor
+ * \author Lars Gullik Bjnnes
  *
- *           Copyright 1995 Matthias Ettrich
- *           Copyright 1995-2001 The LyX Team.
- *
- * ====================================================== */
+ * Full author contact details are available in file CREDITS.
+ */
 
 #ifndef SPACING_H
 #define SPACING_H
 
 #include <iosfwd>
+#include <string>
 
-#include "LString.h"
 
 ///
 class Spacing {
@@ -33,12 +33,12 @@ public:
 		Default
 	};
 	///
-	Spacing() : space(Default), value(1.0) {}
+	Spacing() : space(Default), value("1.0") {}
 	///
-	Spacing(Spacing::Space sp, float val = 1.0) {
+	Spacing(Spacing::Space sp, double val = 1.0) {
 		set(sp, val);
 	}
-	Spacing(Spacing::Space sp, string const & val) {
+	Spacing(Spacing::Space sp, std::string const & val) {
 		set(sp, val);
 	}
 	///
@@ -46,27 +46,29 @@ public:
 		return space == Default;
 	}
 	///
-	float getValue() const;
+	std::string const getValueAsString() const;
+	///
+	double getValue() const;
 	///
 	Spacing::Space getSpace() const { return space; }
 	///
-	void set(Spacing::Space sp, float val = 1.0);
+	void set(Spacing::Space sp, double val = 1.0);
 	///
-	void set(Spacing::Space sp, string const & val) ;
+	void set(Spacing::Space sp, std::string const & val);
 	///
 	void writeFile(std::ostream &, bool para = false) const;
 	///
-	string const writeEnvirBegin() const;
+	std::string const writeEnvirBegin() const;
 	///
-	string const writeEnvirEnd() const;
+	std::string const writeEnvirEnd() const;
 
 private:
 	///
 	Space space;
 	///
-	float value;
+	std::string value;
 	/// names of line spacing
-	static string const spacing_string[];
+	static std::string const spacing_string[];
 };
 
 
@@ -75,7 +77,7 @@ inline
 bool operator==(Spacing const & a, Spacing const & b)
 {
 	return a.getSpace() == b.getSpace()
-		&& a.getValue() == b.getValue();
+		&& a.getValueAsString() == b.getValueAsString();
 }
 
 ///
