@@ -7,24 +7,21 @@
  * \author unknown
  * \author John Levon
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 #ifndef WORKAREA_H
 #define WORKAREA_H
 
 #include "frontends/key_state.h"
-#include "frontends/LyXKeySym.h"
-#include "funcrequest.h"
 
-#include <boost/signals/signal0.hpp>
-#include <boost/signals/signal1.hpp>
-#include <boost/signals/signal2.hpp>
-#include <boost/signals/signal3.hpp>
+#include <boost/shared_ptr.hpp>
+#include <boost/signal.hpp>
 
-#include <utility>
 
 class Painter;
+class FuncRequest;
+class LyXKeySym;
 
 /**
  * The work area class represents the widget that provides the
@@ -35,6 +32,7 @@ class Painter;
  */
 class WorkArea {
 public:
+	typedef boost::shared_ptr<LyXKeySym> LyXKeySymPtr;
 
 	WorkArea() {}
 
@@ -60,24 +58,22 @@ public:
 	/// a selection exists
 	virtual void haveSelection(bool) const = 0;
 	/// get the X clipboard contents
-	virtual string const getClipboard() const = 0;
+	virtual std::string const getClipboard() const = 0;
 	/// fill the clipboard
-	virtual void putClipboard(string const &) const = 0;
+	virtual void putClipboard(std::string const &) const = 0;
 
 	/// work area dimensions have changed
-	boost::signal0<void> workAreaResize;
+	boost::signal<void()> workAreaResize;
 	/// the scrollbar has changed
-	boost::signal1<void, int> scrollDocView;
+	boost::signal<void(int)> scrollDocView;
 	/// a key combination has been pressed
-	boost::signal2<void, LyXKeySymPtr, key_modifier::state> workAreaKeyPress;
+	boost::signal<void(LyXKeySymPtr, key_modifier::state)> workAreaKeyPress;
 	/// some mouse event
-	boost::signal1<void, FuncRequest> dispatch;
-	/// used by drag-and-drop
-	boost::signal1<bool, FuncRequest> viewDispatch;
+	boost::signal<void(FuncRequest)> dispatch;
 	/// emitted when an X client has requested our selection
-	boost::signal0<void> selectionRequested;
+	boost::signal<void()> selectionRequested;
 	/// emitted when another X client has stolen our selection
-	boost::signal0<void> selectionLost;
+	boost::signal<void()> selectionLost;
 };
 
 #endif // WORKAREA_H
