@@ -6,22 +6,25 @@
  *
  * \author Kalle Dalheimer
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 #ifndef FORMABOUT_H
 #define FORMABOUT_H
 
-#include "Qt2Base.h"
+#include "QDialogView.h"
+
+namespace lyx {
+namespace frontend {
 
 class QAboutDialog;
 class ControlAboutlyx;
 
 class QAbout
-	: public Qt2CB<ControlAboutlyx, Qt2DB<QAboutDialog> >
+	: public QController<ControlAboutlyx, QView<QAboutDialog> >
 {
 public:
-	QAbout();
+	QAbout(Dialog &);
 private:
 	/// not needed
 	virtual void apply() {}
@@ -31,4 +34,7 @@ private:
 	virtual void build_dialog();
 };
 
+} // namespace frontend
+} // namespace lyx
+
 #endif // FORMABOUT_H
