@@ -6,18 +6,17 @@
  *
  * \author Kalle Dalheimer
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 #ifndef QABOUTDIALOG_H
 #define QABOUTDIALOG_H
 
-#ifdef __GNUG__
-#pragma interface
-#endif
-
 #include "ui/QAboutDialogBase.h"
 
+namespace lyx {
+namespace frontend {
+
 class QAboutDialog : public QAboutDialogBase {
 	Q_OBJECT
 public:
@@ -26,4 +25,7 @@ public:
 	~QAboutDialog();
 };
 
+} // namespace frontend
+} // namespace lyx
+
 #endif // QABOUTDIALOG_H
