@@ -6,28 +6,31 @@
  *
  * \author John Levon
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 #ifndef QEXTERNAL_H
 #define QEXTERNAL_H
 
-#ifdef __GNUG__
-#pragma interface
-#endif
+#include "QDialogView.h"
+#include <map>
 
-#include "Qt2Base.h"
+namespace lyx {
+namespace frontend {
 
 class ControlExternal;
 class QExternalDialog;
 
+
 class QExternal
-	: public Qt2CB<ControlExternal, Qt2DB<QExternalDialog> >
+	: public QController<ControlExternal, QView<QExternalDialog> >
 {
 public:
 	friend class QExternalDialog;
 
-	QExternal();
+	QExternal(Dialog &);
+
+	typedef std::map<std::string, QString> MapType;
 
 private:
 	/// Apply changes
@@ -37,8 +40,15 @@ private:
 	/// build the dialog
 	virtual void build_dialog();
 
-	/// get the right helptext
-	string const & helpText();
+	/// Helper function called when the template is changed.
+	void updateTemplate();
+	/// get bounding box from file
+	void getBB();
+
+	MapType extra_;
 };
 
+} // namespace frontend
+} // namespace lyx
+
 #endif // QEXTERNAL_H
