@@ -6,26 +6,28 @@
  *
  * \author John Levon
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 #ifndef QINCLUDE_H
 #define QINCLUDE_H
 
-#include "Qt2Base.h"
+#include "QDialogView.h"
+
+namespace lyx {
+namespace frontend {
 
 class ControlInclude;
 class QIncludeDialog;
 
 ///
-class QInclude
-	: public Qt2CB<ControlInclude, Qt2DB<QIncludeDialog> >
+class QInclude : public QController<ControlInclude, QView<QIncludeDialog> >
 {
 public:
 	///
 	friend class QIncludeDialog;
 	///
-	QInclude();
+	QInclude(Dialog &);
 protected:
 	virtual bool isValid();
 private:
@@ -43,4 +45,7 @@ private:
 	void browse();
 };
 
+} // namespace frontend
+} // namespace lyx
+
 #endif // QINCLUDE_H
