@@ -7,7 +7,7 @@
  * \author John Levon
  * \author Edwin Leuven
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 #ifndef QPARAGRAPHDIALOG_H
@@ -15,6 +15,9 @@
 
 #include "ui/QParagraphDialogBase.h"
 
+namespace lyx {
+namespace frontend {
+
 class QParagraph;
 
 class QParagraphDialog : public QParagraphDialogBase {
@@ -27,9 +30,10 @@ private:
 	QParagraph * form_;
 protected slots:
 	void change_adaptor();
-	void enableAbove(int);
-	void enableBelow(int);
 	void enableLinespacingValue(int);
 };
 
+} // namespace frontend
+} // namespace lyx
+
 #endif // QPARAGRAPHDIALOG_H
