@@ -7,7 +7,7 @@
  * \author unknown
  * \author John Levon
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 #ifndef QWORKAREA_H
@@ -17,9 +17,11 @@
 #include "QLPainter.h"
 #include "QContentPane.h"
 
-#include <qwidget.h>
 #include <qscrollbar.h>
-#include <qpixmap.h>
+
+class LyXView;
+class QPixmap;
+class QWidget;
 
 
 /**
@@ -33,7 +35,7 @@ class QWorkArea : public WorkArea, public QWidget {
 public:
 	friend class QContentPane;
 
-	QWorkArea(int x, int y, int w, int h);
+	QWorkArea(LyXView & owner, int w, int h);
 
 	virtual ~QWorkArea();
 	/// return this widget's painter
@@ -48,10 +50,9 @@ public:
 	/// a selection exists
 	virtual void haveSelection(bool) const;
 	///
-	virtual string const getClipboard() const;
+	virtual std::string const getClipboard() const;
 	///
-	virtual void putClipboard(string const &) const;
-
+	virtual void putClipboard(std::string const &) const;
 	///
 	virtual void dragEnterEvent(QDragEnterEvent * event);
 	///
