@@ -7,15 +7,16 @@
  * \author Edwin Leuven
  * \author Angus Leeming
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 #ifndef FORMABOUTLYX_H
 #define FORMABOUTLYX_H
 
-#include "FormBase.h"
+#include "FormDialogView.h"
 
-#include <boost/scoped_ptr.hpp>
+namespace lyx {
+namespace frontend {
 
 class ControlAboutlyx;
 struct FD_aboutlyx;
@@ -25,10 +26,11 @@ struct FD_aboutlyx_license;
 
 /** This class provides an XForms implementation of the FormAboutlyx Dialog.
  */
-class FormAboutlyx : public FormCB<ControlAboutlyx, FormDB<FD_aboutlyx> > {
+class FormAboutlyx
+	: public FormController<ControlAboutlyx, FormView<FD_aboutlyx> > {
 public:
 	///
-	FormAboutlyx();
+	FormAboutlyx(Dialog &);
 
 private:
 	/// not needed.
@@ -46,5 +48,7 @@ private:
 	boost::scoped_ptr<FD_aboutlyx_license> license_;
 };
 
+} // namespace frontend
+} // namespace lyx
 
 #endif // FORMABOUTLYX_H
