@@ -1,29 +1,33 @@
 // -*- C++ -*-
 /**
- * \file xforms/FormCitation.h
+ * \file FormCitation.h
  * This file is part of LyX, the document processor.
  * Licence details can be found in the file COPYING.
  *
  * \author Angus Leeming
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 #ifndef FORMCITATION_H
 #define FORMCITATION_H
 
-#include "FormBase.h"
+#include "FormDialogView.h"
 #include "biblio.h"
 
-/** This class provides an XForms implementation of the Citation Dialog.
- */
+namespace lyx {
+namespace frontend {
+
 class ControlCitation;
 struct FD_citation;
 
-class FormCitation : public FormCB<ControlCitation, FormDB<FD_citation> > {
+/** This class provides an XForms implementation of the Citation Dialog.
+ */
+class FormCitation
+	: public FormController<ControlCitation, FormView<FD_citation> > {
 public:
 	///
-	FormCitation();
+	FormCitation(Dialog &);
 private:
 	///
 	enum State {
@@ -48,16 +52,19 @@ private:
 	void findBiblio(biblio::Direction const dir);
 
 	///
-	void updateBrowser(FL_OBJECT *, std::vector<string> const &) const;
+	void updateBrowser(FL_OBJECT *, std::vector<std::string> const &) const;
 	///
 	void setBibButtons(State) const;
 	///
 	void setCiteButtons(State) const;
 
-	///
-	std::vector<string> citekeys;
-	///
-	std::vector<string> bibkeys;
+	/// used keys (left panel)
+	std::vector<std::string> citekeys;
+	/// available keys (right panel)
+	std::vector<std::string> bibkeys;
 };
 
+} // namespace frontend
+} // namespace lyx
+
 #endif // FORMCITATION_H
