@@ -1,19 +1,21 @@
 // -*- C++ -*-
 /**
- * \file xforms/FormTabularCreate.h
- * Copyright 1995 Matthias Ettrich
+ * \file FormTabularCreate.h
  * This file is part of LyX, the document processor.
  * Licence details can be found in the file COPYING.
  *
  * \author Jrgen Vigna
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 #ifndef FORMTABULARCREATE_H
 #define FORMTABULARCREATE_H
 
-#include "FormBase.h"
+#include "FormDialogView.h"
+
+namespace lyx {
+namespace frontend {
 
 class ControlTabularCreate;
 struct FD_tabular_create;
@@ -22,10 +24,10 @@ struct FD_tabular_create;
     Dialog.
  */
 class FormTabularCreate :
-	public FormCB<ControlTabularCreate, FormDB<FD_tabular_create> > {
+	public FormController<ControlTabularCreate, FormView<FD_tabular_create> > {
 public:
 	///
-	FormTabularCreate();
+	FormTabularCreate(Dialog &);
 private:
 	/// Apply from dialog
 	virtual void apply();
@@ -35,4 +37,7 @@ private:
 	virtual void update() {};
 };
 
+} // namespace frontend
+} // namespace lyx
+
 #endif // FORMTABULARCREATE
