@@ -6,7 +6,7 @@
  *
  * \author Angus Leeming
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 /* Tooltips for xforms. xforms 0.89 supports them directly, but 0.88 needs
@@ -18,23 +18,23 @@
 #ifndef TOOLTIPS_H
 #define TOOLTIPS_H
 
-#include "LString.h"
-
-#include <boost/utility.hpp>
-#include <boost/signals/signal0.hpp>
+#include <boost/signal.hpp>
 #include <boost/signals/trackable.hpp>
 
 #include "forms_fwd.h" // Can't forward-declare FL_OBJECT
 
 #include <map>
 
+namespace lyx {
+namespace frontend {
+
 class Tooltips : boost::noncopyable, public boost::signals::trackable {
 public:
 	///
 	Tooltips();
 
 	/// Initialise a tooltip for this ob.
-	void init(FL_OBJECT * ob, string const & tip);
+	void init(FL_OBJECT * ob, std::string const & tip);
 
 	/// Are the tooltips on or off?
 	static bool enabled() { return enabled_; }
@@ -55,12 +55,15 @@ private:
 	/** Once enabled_ is changed, then this signal is emitted to update
 	 *  all the tooltips.
 	 */
-	static boost::signal0<void> toggled;
+	static boost::signal<void()> toggled;
 
 	/// The tooltips are stored so that they can be turned on and off.
-	typedef std::map<FL_OBJECT *, string> TooltipsMap;
+	typedef std::map<FL_OBJECT *, std::string> TooltipsMap;
 
 	TooltipsMap tooltipsMap;
 };
 
+} // namespace frontend
+} // namespace lyx
+
 #endif // TOOLTIPS_H
