@@ -7,17 +7,12 @@
  * \author Jrgen Vigna
  * \author Lars Gullik Bjnnes
  *
- * Full author contact details are available in file CREDITS
+ * Full author contact details are available in file CREDITS.
  */
 
 #ifndef INSETFOOT_H
 #define INSETFOOT_H
 
-#ifdef __GNUG__
-#pragma interface
-#endif
-
-#include "LaTeXFeatures.h"
 #include "insetfootlike.h"
 
 /** The footnote inset
@@ -28,17 +23,19 @@ public:
 	///
 	InsetFoot(BufferParams const &);
 	///
-	InsetFoot(InsetFoot const &, bool same_id = false);
+	InsetBase::Code lyxCode() const { return InsetBase::FOOT_CODE; }
 	///
-	Inset * clone(Buffer const &, bool same_id = false) const;
+	int latex(Buffer const &, std::ostream &,
+		  OutputParams const &) const;
 	///
-	Inset::Code lyxCode() const { return Inset::FOOT_CODE; }
+	int docbook(Buffer const &, std::ostream &,
+		    OutputParams const & runparams) const;
 	///
-	int latex(Buffer const *, std::ostream &, bool fragile, bool fp) const;
-	///
-	int docbook(Buffer const *, std::ostream &, bool mixcont) const;
-	///
-	string const editMessage() const;
+	std::string const editMessage() const;
+protected:
+	InsetFoot(InsetFoot const &);
+private:
+	virtual std::auto_ptr<InsetBase> doClone() const;
 };
 
 #endif
